from autohandshake.src.Pages.LoginPage import LoginPage
from autohandshake.src.Pages.Page import Page
from autohandshake.src.Pages.InsightsPage import InsightsPage, FileType
from autohandshake.src.Pages.AccessRequestPage import AccessRequestPage, RequestStatus
from autohandshake.src.Pages.AppointmentCalendarPage import AppointmentCalendarPage
from autohandshake.src.Pages.InterviewSchedulePage import InterviewSchedulePage

from autohandshake.src.Pages.SchoolSettings import *
